@@ -29,15 +29,23 @@
 G_BEGIN_DECLS
 
 #ifdef LIBGD_GTK_HACKS
-# include <libgd/gd-create-symbolic-icon.h>
+# include <libgd/gd-icon-utils.h>
 # include <libgd/gd-entry-focus-hack.h>
 #endif
 
+#ifdef LIBGD__VIEW_COMMON
+# include <libgd/gd-main-view-generic.h>
+# include <libgd/gd-styled-text-renderer.h>
+# include <libgd/gd-toggle-pixbuf-renderer.h>
+# include <libgd/gd-two-lines-renderer.h>
+#endif
+
 #ifdef LIBGD_MAIN_ICON_VIEW
 # include <libgd/gd-main-icon-view.h>
-# include <libgd/gd-main-view-generic.h>
-# include <libgd/gd-toggle-pixbuf-renderer.h>
-# include <libgd/gd-two-lines-renderer.h>
+#endif
+
+#ifdef LIBGD_MAIN_VIEW
+# include <libgd/gd-main-view.h>
 #endif
 
 #ifdef LIBGD_MAIN_TOOLBAR
@@ -52,6 +60,14 @@ G_BEGIN_DECLS
 # include <libgd/gd-tagged-entry.h>
 #endif
 
+#ifdef LIBGD_NOTIFICATION
+# include <libgd/gd-notification.h>
+#endif
+
+#ifdef LIBGD_REVEALER
+# include <libgd/gd-revealer.h>
+#endif
+
 G_END_DECLS
 
 #endif /* __GD_H__ */
