@@ -66,6 +66,8 @@ struct _LogviewWindowPrivate {
   gulong monitor_id;
   guint search_timeout_id;
 
+  GCancellable *read_cancellable;
+
   guint filter_merge_id;
   GList *active_filters;
   gboolean matches_only;
@@ -255,14 +257,32 @@ logview_set_fontsize (LogviewWindow *logview, gboolean store)
   }
 }
 
+static void
+logview_set_search_visible (LogviewWindow *window,
+                            gboolean visible)
+{
+  g_action_group_change_action_state (G_ACTION_GROUP (window), "search",
+                                      g_variant_new_boolean (visible));
+}
+
+static void
+logview_clear_active_log_state (LogviewWindow *window,
+                                LogviewLog *old_log)
+{
+  if (window->priv->monitor_id) {
+    g_signal_handler_disconnect (old_log, window->priv->monitor_id);
+    window->priv->monitor_id = 0;
+  }
+
+  logview_set_search_visible (window, FALSE);
+}
+
 static void
 findbar_close_cb (LogviewFindbar *findbar,
                   gpointer user_data)
 {
   LogviewWindow *window = user_data;
-
-  g_action_group_change_action_state (G_ACTION_GROUP (window), "search",
-                                      g_variant_new_boolean (FALSE));
+  logview_set_search_visible (window, FALSE);
 }
 
 static void
@@ -643,13 +663,30 @@ loglist_day_cleared_cb (LogviewLoglist *loglist,
   logview_update_toolbar (logview, active_log, NULL);
 }
 
+static void
+logview_window_schedule_log_read (LogviewWindow *window,
+                                  LogviewLog *log)
+{
+  if (window->priv->read_cancellable != NULL) {
+    g_cancellable_cancel (window->priv->read_cancellable);
+    g_clear_object (&window->priv->read_cancellable);
+  }
+
+  window->priv->read_cancellable = g_cancellable_new ();
+  logview_log_read_new_lines (log,
+                              window->priv->read_cancellable,
+                              (LogviewNewLinesCallback) read_new_lines_cb,
+                              window);
+}
+
 static void
 log_monitor_changed_cb (LogviewLog *log,
                         gpointer user_data)
 {
+  LogviewWindow *window = user_data;
+
   /* reschedule a read */
-  logview_log_read_new_lines (log, (LogviewNewLinesCallback) read_new_lines_cb,
-                              user_data);
+  logview_window_schedule_log_read (window, log);
 }
 
 static void
@@ -685,10 +722,12 @@ read_new_lines_cb (LogviewLog *log,
   gsize len;
 
   if (error != NULL) {
-    primary = g_strdup_printf (_("Can't read from \"%s\""),
-                               logview_log_get_display_name (log));
-    logview_window_add_error (window, primary, error->message);
-    g_free (primary);
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+      primary = g_strdup_printf (_("Can't read from \"%s\""),
+                                 logview_log_get_display_name (log));
+      logview_window_add_error (window, primary, error->message);
+      g_free (primary);
+    }
 
     return;
   }
@@ -742,11 +781,6 @@ read_new_lines_cb (LogviewLog *log,
 
   paint_timestamps (buffer, old_line_count, new_days);
 
-  if (window->priv->monitor_id == 0) {
-    window->priv->monitor_id = g_signal_connect (log, "log-changed",
-                                                 G_CALLBACK (log_monitor_changed_cb), window);
-  }
-
   logview_update_toolbar (window, log, NULL);
   logview_loglist_update_lines (LOGVIEW_LOGLIST (window->priv->loglist), log);
 }
@@ -761,16 +795,9 @@ active_log_changed_cb (LogviewManager *manager,
   const char **lines;
   GtkTextBuffer *buffer;
 
-  findbar_close_cb (LOGVIEW_FINDBAR (window->priv->find_bar),
-                    window);
-
+  logview_clear_active_log_state (window, old_log);
   logview_update_toolbar (window, log, NULL);
 
-  if (window->priv->monitor_id) {
-    g_signal_handler_disconnect (old_log, window->priv->monitor_id);
-    window->priv->monitor_id = 0;
-  }
-
   lines = logview_log_get_cached_lines (log);
   buffer = gtk_text_buffer_new (window->priv->tag_table);
 
@@ -791,13 +818,13 @@ active_log_changed_cb (LogviewManager *manager,
     paint_timestamps (buffer, 1, logview_log_get_days_for_cached_lines (log));
   }
 
+  window->priv->monitor_id =
+    g_signal_connect (log, "log-changed",
+                      G_CALLBACK (log_monitor_changed_cb), window);
+
   if (lines == NULL || logview_log_has_new_lines (log)) {
     /* read the new lines */
-    logview_log_read_new_lines (log, (LogviewNewLinesCallback) read_new_lines_cb, window);
-  } else {
-    /* start now monitoring the log for changes */
-    window->priv->monitor_id = g_signal_connect (log, "log-changed",
-                                                 G_CALLBACK (log_monitor_changed_cb), window);
+    logview_window_schedule_log_read (window, log);
   }
 
   /* we set the buffer to the view anyway;
@@ -908,6 +935,11 @@ logview_window_finalize (GObject *object)
 {
   LogviewWindow *logview = LOGVIEW_WINDOW (object);
 
+  if (logview->priv->read_cancellable != NULL) {
+    g_cancellable_cancel (logview->priv->read_cancellable);
+    g_clear_object (&logview->priv->read_cancellable);
+  }
+
   g_clear_object (&logview->priv->filters_placeholder);
   pango_font_description_free (logview->priv->monospace_description);
 
@@ -1026,7 +1058,7 @@ action_close (GSimpleAction *action,
 {
   LogviewWindow *logview = user_data;
 
-  findbar_close_cb (LOGVIEW_FINDBAR (logview->priv->find_bar), logview);
+  logview_set_search_visible (logview, FALSE);
   logview_manager_close_active_log (logview->priv->manager);
 }
 
